package org.jvnet.transcoder.maven2.util;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.Date;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;
import java.util.Map.Entry;

public class PropertyUtils {

  public static Map toLowerCase(Map draftMap, Locale locale) {
    final Map map = new TreeMap();
    for (Iterator iterator = draftMap.entrySet().iterator(); iterator.hasNext();) {
      final Entry entry = (Entry) iterator.next();
      final String key = (String) entry.getKey();
      final String value = (String) entry.getValue();
      map.put(key.toLowerCase(), value);
    }
    return map;
  }

  public static Map load(InputStream is) throws IOException {
    final Properties properties = new Properties();
    properties.load(is);
    return toMap(properties);
  }

  public static Map toMap(Properties properties) {
    final Map map = new TreeMap();
    for (Iterator iterator = properties.entrySet().iterator(); iterator.hasNext();) {
      final Entry entry = (Entry) iterator.next();
      final String key = (String) entry.getKey();
      final String value = (String) entry.getValue();
      map.put(key, value);
    }
    return map;
  }

  public static void store(Map properties, OutputStream out, String comments) throws IOException {
    final BufferedWriter awriter;
    awriter = new BufferedWriter(new OutputStreamWriter(out, "8859_1"));
    if (comments != null) {
      writeln(awriter, "#" + comments);
    }
    writeln(awriter, "#" + new Date().toString());
    for (Iterator iterator = properties.keySet().iterator(); iterator.hasNext();) {
      final String key = (String) iterator.next();
      final String value = (String) properties.get(key);
      writeln(awriter, escape(key, true) + "=" + escape(value, false));
    }
    awriter.flush();
  }

  public static String escape(String theString, boolean escapeSpace) {
    final int len = theString.length();
    final StringBuffer outBuffer = new StringBuffer(len * 2);

    for (int x = 0; x < len; x++) {
      char aChar = theString.charAt(x);
      // Handle common case first, selecting largest block that
      // avoids the specials below
      if ((aChar > 61) && (aChar < 127)) {
        if (aChar == '\\') {
          outBuffer.append('\\');
          outBuffer.append('\\');
          continue;
        }
        outBuffer.append(aChar);
        continue;
      }
      switch (aChar) {
        case ' ':
          if (x == 0 || escapeSpace)
            outBuffer.append('\\');
          outBuffer.append(' ');
          break;
        case '\t':
          outBuffer.append('\\');
          outBuffer.append('t');
          break;
        case '\n':
          outBuffer.append('\\');
          outBuffer.append('n');
          break;
        case '\r':
          outBuffer.append('\\');
          outBuffer.append('r');
          break;
        case '\f':
          outBuffer.append('\\');
          outBuffer.append('f');
          break;
        case '=': // Fall through
        case ':': // Fall through
        case '#': // Fall through
        case '!':
          outBuffer.append('\\');
          outBuffer.append(aChar);
          break;
        default:
          if ((aChar < 0x0020) || (aChar > 0x007e)) {
            outBuffer.append('\\');
            outBuffer.append('u');
            outBuffer.append(toHex((aChar >> 12) & 0xF));
            outBuffer.append(toHex((aChar >> 8) & 0xF));
            outBuffer.append(toHex((aChar >> 4) & 0xF));
            outBuffer.append(toHex(aChar & 0xF));
          }
          else {
            outBuffer.append(aChar);
          }
      }
    }
    return outBuffer.toString();
  }

  public static void writeln(BufferedWriter bw, String s) throws IOException {
    bw.write(s);
    bw.newLine();
  }

  public static char toHex(int nibble) {
    return HEX_DIGITS[(nibble & 0xF)];
  }

  public static final char[] HEX_DIGITS = {
      '0',
      '1',
      '2',
      '3',
      '4',
      '5',
      '6',
      '7',
      '8',
      '9',
      'A',
      'B',
      'C',
      'D',
      'E',
      'F' };
}
